from datetime import datetime
import random
from ameliabot.logger import logging


try:
    import psycopg2
except ModuleNotFoundError:
    logging.error("Please install psycopg2.")
    logging.error("Please see README.md for more information.")
    raise SystemExit


class Quote:
    def __init__(self, database, user, password):
        self.conn = psycopg2.connect(database=database, user=user, password=password)
        self.num_quotes = self._get_num_quotes()

    def insert(self, owner, submitter, text):
        cur = self.conn.cursor()
        text = text.replace("'", "''")
        cur.execute('''
            INSERT INTO quotes (owner, submitter, text, timestamp)
            VALUES ('{}', '{}', E'{}', current_timestamp)'''.format(
                owner, submitter, text))
        self.conn.commit()
        cur.close()
        self.num_quotes += 1

    def get(self, arg=None):
        if arg:
            cur = self.conn.cursor()
            ret = "SELECT number, text, timestamp FROM quotes WHERE "
            try:
                arg = int(arg)
                if arg > self.num_quotes:
                    return "No quote matching that number."
                ret += "number = {}".format(arg)
            except ValueError:
                ret += "text like '%{}%'".format(arg.lower())

            cur.execute(ret)
            quotes = cur.fetchall()
            cur.close()

            return self._format(quotes)
        else:
            return self.get(random.randint(0, self.num_quotes))

    def _get_num_quotes(self):
        cur = self.conn.cursor()
        cur.execute("SELECT COUNT(id) FROM quotes")
        return cur.fetchone()[0]

    def _format(self, quotes):
        if len(quotes) >= 1:
            data = quotes[random.randint(0, len(quotes)-1)]
        else:
            try:
                data = quotes[0]
            except IndexError:
                return "No quote."
        return "{}. {}, {}".format(
            data[0], data[1], datetime.strftime(data[2], '%Y'))
